/* $Id: sexec.C,v 1.2 1999/07/21 18:58:22 kaminsky Exp $ */

/*
 *
 * Copyright (C) 1999 Michael Kaminsky (kaminsky@lcs.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "sfsmisc.h"
#include "parseopt.h"
#include <pwd.h>

int
main (int argc, char **argv)
{
  setprogname (argv[0]);
  sfsconst_init ();

  str program;

  if (argc < 2) {
    warnx << "usage: " << progname << " auxgid program [ args ... ]\n";
    exit (1);
  }
  else if (argc == 2) {
    struct passwd *pe;
    if ((pe = getpwuid (getuid ())))
      program = pe->pw_shell;
    else
      program = "/bin/sh";
  }

  str newaid = fix_exec_path ("newaid");

  vector<char *> av;
  av.push_back (const_cast<char *> (newaid.cstr ()));

  for (int i = 1; i < argc; i++)
    av.push_back (const_cast<char *> (argv[i]));
  if (program)
    av.push_back (const_cast<char *> (program.cstr ()));

  av.push_back (NULL);

  /* 
     The SFS libraries use asynchronous IO which some programs don't
     like.  Thus, we remove the O_NONBLOCK flag from stdin/stdout.
  */
  make_sync (0);
  make_sync (1);

  execvp (av[0], av.base ());

  /* shouldn't be reached */
  warn << "exec: " << strerror (errno) << "\n";
  return 1;
}
